<?php

namespace Bydedi\Template\Smarty;

use Bydedi\Template\AbstractPlugin;
use Symfony\Component\Templating\EngineInterface;
use Symfony\Component\Templating\TemplateNameParserInterface;
use Symfony\Component\HttpFoundation\Response;
use Smarty;

/**
 * Symfony Templating bridge to Smarty.
 */
class SmartyEngine implements EngineInterface
{
    /**
     * The Smarty template engine instance.
     */
    protected $smarty;

    /**
     * A TemplateNameParserInterface instance
     */
    protected $parser;

    /**
     * Build the Smarty engine.
     *
     * @param array $options
     *   An array of parameters used to set up the Smarty
     *   configuration. Available configuration values include:
     *     - cache_dir
     *     - compile_dir
     *     - config_dir
     *     - plugins_dir
     *     - template_dir
     * @param TemplateNameParserInterface $parser
     *   A TemplateNameParserInterface instance
     */
    public function __construct(TemplateNameParserInterface $parser, array $options = array())
    {
        // The parser to use.
        $this->parser = $parser;

        // Create the Smarty template engine.
        $this->smarty = new Smarty();

        // Set any of the required configuration.
        if (isset($options['cache_dir'])) {
            $this->smarty->setCacheDir($options['cache_dir']);
        }
        if (isset($options['compile_dir'])) {
            $this->smarty->setCompileDir($options['compile_dir']);
        }
        if (isset($options['config_dir'])) {
            $this->smarty->setConfigDir($options['config_dir']);
        }
        if (isset($options['plugins_dir'])) {
            $this->smarty->addPluginsDir($options['plugins_dir']);
        }
        if (isset($options['template_dir'])) {
            $this->smarty->setTemplateDir($options['template_dir']);
        }
    }

    /**
     * Renders a template.
     *
     * @param string|TemplateReferenceInterface $name       A template name or a TemplateReferenceInterface instance
     * @param array                             $parameters An array of parameters to pass to the template
     *
     * @return string The evaluated template as a string
     *
     * @throws \RuntimeException if the template cannot be rendered
     *
     * @api
     */
    public function render($name, array $parameters = array())
    {
        $this->smarty->assign($parameters);

        // Render the template using Smarty.
        $output = $this->smarty->fetch($this->nameToString($name));

        return trim($output);
    }

    /**
     * @param $view
     * @param array $parameters
     * @param Response $response
     *
     * @return Response
     */
    public function renderResponse($view, array $parameters = array(), Response $response = null)
    {
        if (null === $response) {
            $response = new Response();
        }

        $response->setContent($this->render($view, $parameters));

        return $response;
    }

    /**
     * Returns true if the template exists.
     *
     * @param string|TemplateReferenceInterface $name A template name or a TemplateReferenceInterface instance
     *
     * @return bool    true if the template exists, false otherwise
     *
     * @throws \RuntimeException if the engine cannot handle the template name
     *
     * @api
     */
    public function exists($name)
    {
        return $this->smarty->templateExists($this->nameToString($name));
    }

    /**
     * Returns true if this class is able to render the given template.
     *
     * @param string|TemplateReferenceInterface $name A template name or a TemplateReferenceInterface instance
     *
     * @return bool    true if this class supports the given template, false otherwise
     *
     * @api
     */
    public function supports($name)
    {
        $template = $this->parser->parse($name);

        return 'smarty' === $template->get('engine');
    }

    public function registerClass($class_name, $class_impl)
    {
        $this->smarty->registerClass($class_name, $class_impl);
    }

    /**
     * Converts a template name to string if it is anything other than a string.
     */
    private function nameToString($name)
    {
        return is_string($name) ? $name : $name->toString();
    }

    public function registerPlugins($plugins)
    {
        foreach ($plugins as $plugin) {
            $this->registerPlugin($plugin);
        }
    }

    public function registerPlugin(AbstractPlugin $plugin)
    {
        list($type, $name, $callback) = $plugin->register();
        $this->smarty->registerPlugin($type, $name, $callback);
    }

    public function assign($tpl_var, $value)
    {
        $this->smarty->assign($tpl_var, $value);
    }

    public function clearCache()
    {
        $this->smarty->clearAllCache();
    }

    public function clearCompiledTemplate()
    {
        $this->smarty->clearCompiledTemplate();
    }
}
